/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <color.hpp>
#include <cmath>
#include <iostream>
#include <sstream>

#include <vec3.hpp>

namespace cpe
{

    color::color()
        :internal_r(0),internal_g(0),internal_b(0)
    {}
    color::color(color_component gray)
        :internal_r(gray),internal_g(gray),internal_b(gray)
    {}
    color::color(color_component r_param,color_component g_param,color_component b_param)
        :internal_r(r_param),internal_g(g_param),internal_b(b_param)
    {}

    color::color(const vec3& c)
        :internal_r(c.x()),internal_g(c.y()),internal_b(c.z())
    {}

    color_component color::r() const
    {
        return internal_r;
    }
    color_component& color::r()
    {
        return internal_r;
    }
    color_component color::g() const
    {
        return internal_g;
    }
    color_component& color::g()
    {
        return internal_g;
    }
    color_component color::b() const
    {
        return internal_b;
    }
    color_component& color::b()
    {
        return internal_b;
    }

    //affichage a l'ecran d'une couleur
    std::ostream& operator<<(std::ostream& stream,const color& c)
    {
        //affiche uniquement "r g b" que l'on concatene dans le flux.
        stream<<c.r()<<" "<<c.g()<<" "<<c.b();
        return stream;
    }



    //tronque chaque composante de couleur entre 0 et 255
    void color::clamp()
    {
        internal_r=std::min(std::max(internal_r,0.0f),1.0f);
        internal_g=std::min(std::max(internal_g,0.0f),1.0f);
        internal_b=std::min(std::max(internal_b,0.0f),1.0f);
    }

    color& operator+=(color& c0,const color& c1)
    {
        c0.r()+=c1.r();
        c0.g()+=c1.g();
        c0.b()+=c1.b();
        return c0;
    }

    color& operator-=(color& c0,const color& c1)
    {
        c0.r()-=c1.r();
        c0.g()-=c1.g();
        c0.b()-=c1.b();
        return c0;
    }

    color operator+(const color& c0,const color& c1)
    {
        color temp=c0;
        temp+=c1;
        return temp;
    }

    color operator-(const color& c0,const color& c1)
    {
        color temp=c0;
        temp-=c1;
        return temp;
    }

    color& operator*=(color& c0,float alpha)
    {
        c0.r()*=alpha;
        c0.g()*=alpha;
        c0.b()*=alpha;
        return c0;
    }

    color& operator/=(color& c0,float alpha)
    {
        if(std::abs(alpha)<1e-5)
        {
            std::ostringstream msg;msg<<"Cannot divide by 0";
            throw exception_color(msg.str(),MACRO_EXCEPTION_PARAMETER);
        }

        c0.r()/=alpha;
        c0.g()/=alpha;
        c0.b()/=alpha;
        return c0;
    }

    color operator*(const color& c0,float alpha)
    {
        color temp=c0;
        temp*=alpha;
        return temp;
    }
    color operator*(float alpha,const color& c0)
    {
        return c0*alpha;
    }

    color operator/(const color& c0,float alpha)
    {
        if(std::abs(alpha)<1e-5)
        {
            std::ostringstream msg;msg<<"Cannot divide by 0";
            throw exception_color(msg.str(),MACRO_EXCEPTION_PARAMETER);
        }

        color temp=c0;
        temp/=alpha;
        return temp;
    }

}
