/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef IMAGE_ZBUFFER_HPP
#define IMAGE_ZBUFFER_HPP

#include <iostream>
#include <vector>
#include <exception_cpe.hpp>


#include <image.hpp>
#include <z_buffer.hpp>

namespace cpe
{

class pos2;

/** \brief Container for picture data. Data are stored as unsigned char [0,255] */
class image_zbuffer: public image
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        image_zbuffer();
        /** \brief create square picture of size NxN
            \param unsigned int N: size (width & height) of the picture
        */
        image_zbuffer(int N);
        /** \brief create picture of size Nx x Ny
            \param unsigned int Nx: picture width
            \param unsigned int Ny: picture height
        */
        image_zbuffer(int Nx,int Ny);


        // ********************************************* //
        // ********************************************* //
        //  Zbuffer access
        // ********************************************* //
        // ********************************************* //

        /** \brief Accessor on a pixel */
        double z(const pos2& index) const;
        /** \brief Accessor on a pixel */
        double& z(const pos2& index);


        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief export depth-buffer into a file */
        void export_z_buffer(const std::string& filename) const;


    private:


        // ********************************************* //
        // ********************************************* //
        //  Internal data
        // ********************************************* //
        // ********************************************* //

        /** \brief internal data storage as std::vector of unsigned char

            Picture is stored as (color_0 , color_1 , ...) vector size is therefore 3 Nx Ny
        */
        z_buffer internal_z_buffer;


};


}

#endif
