/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <line_discrete.hpp>
#include <assert.h>
#include <iostream>
#include <cmath>

namespace cpe
{
void line_discrete::add(const pos2& position)
{
    coordinate.push_back(position);
}
int line_discrete::size() const
{
    return coordinate.size();
}

line_discrete swap_xy(const line_discrete& line)
{
    line_discrete new_line;
    for(int k=0,N=line.size();k<N;++k)
    {
        const pos2& p=line.coordinate[k];
        new_line.add(pos2(p.y(),p.x()));
    }
    return new_line;
}

line_discrete symetry_x(const line_discrete& line)
{
    assert(line.size()>0);

    line_discrete new_line;
    int y0=line.coordinate[0].y();
    for(int k=0,N=line.size();k<N;++k)
    {
        const pos2& p=line.coordinate[k];
        new_line.add(pos2(p.x(),2*y0-p.y()));
    }
    return new_line;
}
line_discrete symetry_y(const line_discrete& line)
{
    assert(line.size()>0);

    line_discrete new_line;
    int x0=line.coordinate[0].x();
    for(int k=0,N=line.size();k<N;++k)
    {
        const pos2& p=line.coordinate[k];
        new_line.add(pos2(2*x0-p.x(),p.y()));
    }
    return new_line;
}

void line_discrete::debug() const
{
    for(int k=0,N=size();k<N;++k)
    {
        std::cout<<k<<"/"<<N<<" : "<<coordinate[k]<<std::endl;
    }
}
line_discrete reverse(const line_discrete& line)
{
    line_discrete new_line;
    for(int k=0,N=line.size();k<N;++k)
        new_line.add(line.coordinate[N-k-1]);
    return new_line;
}

void line_discrete::compute_interpolation()
{
    // ********************************************* //
    // ********************************************* //
    // [TO DO] : Calcul des position relative le long du segment
    //
    // ********************************************* //
    // ********************************************* //

}


}
