/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LINE_DISCRETE_HPP
#define LINE_DISCRETE_HPP

#include <vector>
#include <pos2.hpp>

namespace cpe
{

/** \brief Container for vector of 2D integer coordinates */
struct line_discrete
{
    /** \brief Add a new coordinate in the vector */
    void add(const pos2& position);
    /** \brief Get the size of the coordinate vector */
    int size() const;



    /** \brief Print the values of the coordinate vector */
    void debug() const;

    /** \brief Compute the relative coordinate values between beginning to the end */
    void compute_interpolation();

    /** \brief Storage for the coordinates */
    std::vector<pos2> coordinate;


};

/** \brief Return a new line_discrete with swaped values between x & y */
line_discrete swap_xy(const line_discrete& line);
/** \brief Return a new line_discrete which is symetric with respect to x-coordinate */
line_discrete symetry_x(const line_discrete& line);
/** \brief Return a new line_discrete which is symetric with respect to y-coordinate */
line_discrete symetry_y(const line_discrete& line);
/** \brief Return a new line_discrete in the reverse order (last entry <-> first entry) */
line_discrete reverse(const line_discrete& line);

}

#endif
