/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <line_discrete_algorithm.hpp>

#include <line_discrete.hpp>
#include <pos2.hpp>

#include <assert.h>
#include <cmath>
#include <cstdlib>

namespace cpe
{

line_discrete compute_line(const pos2& u0,const pos2& u1)
{
    int x0=u0.x();
    int x1=u1.x();
    int y0=u0.y();
    int y1=u1.y();

    int dx=x1-x0;
    int dy=y1-y0;

    //line = 1 point
    if(dx==0 && dy==0)
    {
        line_discrete line;
        line.add(u0);
        return line;
    }

    //vertical
    if(dx==0)
    {
        int x=x0;

        if(dy>0)
            return compute_line_vertical(x,y0,y1);
        else
            return reverse(compute_line_vertical(x,y1,y0));
    }

    // ********************************************* //
    // ********************************************* //
    // [TO DO]: Cas de la ligne horizontale
    //
    // ********************************************* //
    // ********************************************* //

    // ********************************************* //
    // ********************************************* //
    // DECOMMENTEZ la partie suivante pour le traitement des diagonale (et de ses symetriques)
    //
    // ********************************************* //
    // ********************************************* //
    //    //diagonal
    //    int abs_dx=dx<0?dx:-dx;
    //    int abs_dy=dy<0?dy:-dy;
    //    if(abs_dx==abs_dy)
    //    {
    //        if(dx>0 && dy>0)
    //            return compute_line_diagonal(x0,y0,x1,y1);
    //        if(dx<0 && dy<0)
    //            return reverse(compute_line_diagonal(x1,y1,x0,y0));
    //        if(dx<0 && dy>0)
    //            return symetry_y(compute_line_diagonal(x0,y0,x0-dx,y1));
    //        if(dx>0 && dy<0)
    //            return symetry_x(compute_line_diagonal(x0,y0,x1,y0-dy));
    //    }




    // ********************************************* //
    // ********************************************* //
    // DECOMMENTEZ la partie suivante pour le traitement des lignes obliques (et de ses symetiques)
    //
    // ********************************************* //
    // ********************************************* //
    //    //bresenham
    //    if(dx>0 && dy>0 && dx>dy)
    //        return compute_bresenham(x0,y0,x1,y1);
    //    if(dx>0 && dy>0 && dx<dy)
    //        return swap_xy(compute_bresenham(y0,x0,y1,x1));
    //    if(dx<0 && dy>0 && -dx>dy)
    //        return symetry_y(compute_bresenham(x0,y0,x0-dx,y1));
    //    if(dx<0 && dy>0 && -dx<dy)
    //        return symetry_y(swap_xy(compute_bresenham(y0,x0,y1,x0-dx)));
    //    if(dx<0 && dy<0 && -dx>-dy)
    //        return reverse(compute_bresenham(x1,y1,x0,y0));
    //    if(dx<0 && dy<0 && -dx<-dy)
    //        return reverse(swap_xy(compute_bresenham(y1,x1,y0,x0)));
    //    if(dx>0 && dy<0 && dx>-dy)
    //        return symetry_x(compute_bresenham(x0,y0,x1,y0-dy));
    //    if(dx>0 && dy<0 && dx<-dy)
    //        return symetry_x(swap_xy(compute_bresenham(y0,x0,y0-dy,x1)));



    std::cout<<"Should never arrive at this point in file"<<__FILE__<<" in function "<<__FUNCTION__<<std::endl;
    abort();
}


line_discrete compute_line_vertical(int x,int y_min,int y_max)
{
    assert(y_min<y_max);

    line_discrete line;

    // ********************************************* //
    // ********************************************* //
    // Algorithm:
    //
    //   Pour y=[y_min,y_max]
    //      Ajouter position (x,y)
    //
    // ********************************************* //
    // ********************************************* //

    for(int y=y_min;y<=y_max;++y)
        line.add(pos2(x,y));

    return line;
}

line_discrete compute_line_horizontal(int x_min,int x_max,int y)
{
    assert(x_min<x_max);

    line_discrete line;

    // ********************************************* //
    // ********************************************* //
    // [TO DO] : CALCUL D'UNE LIGNE VERTICALE
    //
    // Algorithm:
    //
    //   Pour x=[x_min,x_max]
    //      Ajouter position (x,y)
    //
    // ********************************************* //
    // ********************************************* //

    return line;
}

line_discrete compute_line_diagonal(int x0,int y0,int x1,int y1)
{
    assert(x0<x1);
    assert(y0<y1);
    assert(x1-x0==y1-y0);

    line_discrete line;

    // ********************************************* //
    // ********************************************* //
    // [TO DO] : CALCUL D'UNE LIGNE DIAGONALE
    //
    // ********************************************* //
    // ********************************************* //

    return line;
}


line_discrete compute_bresenham(int x0,int y0,int x1,int y1)
{
    line_discrete line;

    int dy=y1-y0;
    int dx=x1-x0;
    assert(dy>0);
    assert(dx>0);
    assert(dx>dy);

    // ********************************************* //
    // ********************************************* //
    // [TO DO] : ALGORITHM DE BRESENHAM
    //
    // ********************************************* //
    // ********************************************* //

    return line;
}






}
