/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LINE_DISCRETE_ALGORITHM_HPP
#define LINE_DISCRETE_ALGORITHM_HPP




namespace cpe
{

class line_discrete;
class pos2;


/** \brief Compute a line_discrete between any positions (u0,u1) */
line_discrete compute_line(const pos2& u0,const pos2& u1);

/** \brief Compute a horizontal line at constant x between (y_min,y_max).
 *  \pre y_min<y_max
 */
line_discrete compute_line_vertical(int x,int y_min,int y_max);

/** \brief Compute a horizontal line at constant y between (x_min,x_max).
 * \pre x_min<x_max
 */
line_discrete compute_line_horizontal(int x_min,int x_max,int y);

/** \brief Compute bresenham algorithm in the first quadrant.
 *  \pre x0<x1 && y0<y1 && (x1-x0)>(y1-y0)
 */
line_discrete compute_bresenham(int x0,int y0,int x1,int y1);

/** \brief Compute diagonal line.
 *  \pre x0<x1 && y0<y1 && x1-x0==y1-y0
 */
line_discrete compute_line_diagonal(int x0,int y0,int x1,int y1);

}

#endif
