/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef POS2_HPP
#define POS2_HPP

#include <iostream>
#include <exception_cpe.hpp>

namespace cpe
{
    /** \brief Container for 2D integer positions (pixel position) */
    class pos2
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor
            \param int x0: x-coordinate
            \param int y0: y-coordinate
        */
        pos2(int x0=0,int y0=0);

        // ********************************************* //
        // ********************************************* //
        //  Coordinate access
        // ********************************************* //
        // ********************************************* //

        /** \brief get x-coordinate */
        int x() const;
        /** \brief get x-coordinate */
        int& x();
        /** \brief get y-coordinate */
        int y() const;
        /** \brief get y-coordinate */
        int& y();


    private:

        // ********************************************* //
        // ********************************************* //
        //  Internal storage
        // ********************************************* //
        // ********************************************* //

        /** \brief internal x-coordinate */
        int internal_x;
        /** \brief internal y-coordinate */
        int internal_y;

    };


    // ********************************************* //
    // ********************************************* //
    //  Operators
    // ********************************************* //
    // ********************************************* //

    /** \brief sum between two positions */
    pos2 operator+(const pos2& u1,const pos2& u2);
    /** \brief diff between two positions */
    pos2 operator-(const pos2& u1,const pos2& u2);
    /** \brief internal sum between two positions */
    pos2& operator+=(pos2& u1,const pos2& u2);
    /** \brief internal diff between two positions */
    pos2& operator-=(pos2& u1,const pos2& u2);

    // ********************************************* //
    // ********************************************* //
    //  I/O
    // ********************************************* //
    // ********************************************* //

    /** \brief standard output */
    std::ostream& operator<<(std::ostream& stream,const pos2& u);



    /** \brief Exception class for p2d */
    class exception_p2d : public exception_cpe
    {
    public:
        /** \brief empty constructor */
        exception_p2d():exception_cpe(){}
        /** \brief constructor with error information */
        exception_p2d(const std::string& msg_param,const std::string& file_param,const std::string& caller_param,const int& line_param):exception_cpe(msg_param,file_param,caller_param,line_param){}
    };
}


#endif
