/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TRIANGLE_SCANLINE_HPP
#define TRIANGLE_SCANLINE_HPP

#include <line_discrete.hpp>
#include <line_discrete_algorithm.hpp>
#include <pos2.hpp>

namespace cpe
{

/** Pointer for triangle scanline */
struct line_pointer
{
    line_pointer(int line_number_data=0,int index_data=0);

    int line_number; //the number of the pointed line (0,1, or 2)
    int index; //the index in the pointed line
};


/** Structure for triangle scanline */
struct triangle_scanline
{
public:

    /** \brief Direct construction from 3 positions (u0,u1,u2) */
    triangle_scanline(const pos2& u0,const pos2& u1,const pos2& u2);

    /** \brief Get coordinate of the discrete line pointed by p */
    const pos2& get_coordinate(const line_pointer& p);

    /** \brief Size of triangle in x direction */
    int size();

    /** \brief storage of the pointer for the first point */
    std::vector<line_pointer> first_point;
    /** \brief storage of the pointer for the last point */
    std::vector<line_pointer> last_point;

    /** \brief storage for the 3 discrete lines of the border of the triangle */
    line_discrete line[3];
};

}

#endif
