/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RENDER_ENGINE_HPP
#define RENDER_ENGINE_HPP


#include <scene.hpp>
#include <shading.hpp>


namespace cpe
{
class image_zbuffer;
class vertex;
class mesh;
class vec3;
class pos2;

/** \brief Renderer */
class render_engine
{
public:

    /** \brief render engine contains an image (reference), parameter of the scene and parameter of shading */
    render_engine(image_zbuffer& im,const scene& scene_parameter,const shading& shading_parameter);

    /** \brief render a triangle on the screen */
    void render(const vertex& v0,const vertex& v1,const vertex& v2);
    /** \brief render a mesh on the screen */
    void render(const mesh& m);

private:

    /** \brief internal storage of the image */
    image_zbuffer& internal_image;
    /** \brief internal storage of the scene */
    scene internal_scene;
    /** \brief internal storage of the shading */
    shading internal_shading;
};


/** \brief Convert real coordinate to image coordinate */
pos2 real_coordinate_to_image_coordinate(const vec3& p,int Nx,int Ny);

}

#endif
