/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef SHADING_HPP
#define SHADING_HPP

#include <color.hpp>
#include <vec3.hpp>
#include <scene.hpp>
#include <vertex.hpp>

namespace cpe
{

/** \brief Container for shading parameters (ambiant,diffuse,specular)*/
struct shading
{
        /** \brief direct constructor for the shading parameters */
        shading(float ambiant=0.1,float diffuse=0.6,float specular=0.4,float specular_exponent=2.0);


        /** \brief ambiant component */
        float ambiant;
        /** \brief diffuse component */
        float diffuse;
        /** \brief specular component */
        float specular;
        /** \brief specular exponent */
        float specular_exponent;

};

/** \brief Apply gouraud shading on the given vertex */
color apply_shading(const vertex& v,const scene& scene3d,const shading& shad);
/** \brief Apply gouraud shading */
color apply_shading(const vec3& p,const vec3& n,const color& c,const scene& scene3d,const shading& shad);

}

#endif
